#!/bin/sh
kernelversion=2.6.24.7-kateos

moduledep() {
    if [ ! -f "$2/lib/modules/$kernelversion/modules.dep" ]; then
	exit 1
    fi
    deps=$(awk 'BEGIN { searched=ARGV[2]; ARGV[2]=""; rc=1 } \
    function modname(filename) { match(filename, /\/([^\/]+)\.k?o(.gz):?$/, ret); return ret[1] } \
                function show() { if (orig == searched) { print dep; orig=""; rc=0; exit } } \
                /^\/lib/ { show(); \
                           orig=modname($1); dep=""; \
                           if ($2) { for (i = 2; i <= NF; i++) { dep=sprintf("%s %s", dep, modname($i)); } } } \
                /^	/ { dep=sprintf("%s %s", dep, modname($1));  } \
                END      { show(); exit(rc) }' $2/lib/modules/$kernelversion/modules.dep $1)
echo "$deps"
}

afterinstall(){

if [ -f /sbin/depmod ]; then
/sbin/depmod -a $kernelversion
fi


if test -f boot/initrd ; then

if test ! -d tmp/mount ; then
mkdir tmp/mount
fi

if file boot/initrd | grep gzip 1>/dev/null 2>/dev/null ; then 
GZIP=1
zcat boot/initrd > tmp/initrd
else
GZIP=0
cat boot/initrd > tmp/initrd
fi

CPIO=0
rcfile=linuxrc

if file tmp/initrd | grep cpio 1>/dev/null 2>/dev/null ; then 
CPIO=1
rcfile=init
fi

if [ "$CPIO" != 1 ] ; then
mount -o loop tmp/initrd tmp/mount
else
( cd tmp/mount ; cat ../initrd | cpio -i -d -H newc --no-absolute-filenames 2>/dev/null )
fi

( cd tmp/mount/lib
for module in *.ko ; do
shortname=$(basename $module .ko)
modfile="$(cd ../../.. ; find lib/modules/$kernelversion/ -name $module.gz -type f)"
rm $module
if [ ! -z $modfile ] ; then
(cd ../../.. ; zcat $modfile > tmp/mount/lib/$module) 

deps=$(moduledep $shortname ../../..)
if [ "$?" = "0" ] ; then
# We must be sure, that we don't need any additional dependencies!
for dep in $deps ; do
if [ ! -f $dep.ko ] ; then
depfile="$(cd ../../.. ; find lib/modules/$kernelversion/ -name $dep.ko.gz -type f)"
    
    if [ ! -z $depfile ] ; then
    (cd ../../.. ; zcat $depfile > tmp/mount/lib/$dep.ko) 

    # Yup, rcfile must be also changed ;)
    (cd ../../.. ; cat tmp/mount/$rcfile | sed -e "s/insmod \/lib\/$module/insmod \/lib\/$dep.ko\ninsmod \/lib\/$module/g" > tmp/mount/$rcfile.new ; mv tmp/mount/$rcfile.new tmp/mount/$rcfile ; chmod +x tmp/mount/$rcfile) 
    fi
fi
done
fi

else
    (cd ../../.. ; cat tmp/mount/$rcfile | sed -e "/$module/d" > tmp/mount/$rcfile.new ; mv tmp/mount/$rcfile.new tmp/mount/$rcfile ; chmod +x tmp/mount/$rcfile) 
fi # modfile

done
)

if [ "$CPIO" != 1 ] ; then
umount tmp/mount
else
rm -f tmp/initrd
( cd tmp/mount ; find . | cpio -o -H newc > ../initrd 2>/dev/null )
fi
rm -R tmp/mount

mv boot/initrd boot/initrd.old

if [ "$GZIP" = "1" ] ; then
gzip -9 tmp/initrd
mv tmp/initrd.gz boot/initrd
else
mv tmp/initrd boot/initrd
fi

fi


}

beforeremove(){
true
}

case "$1" in
'--afterinstall')
afterinstall
;;
'--beforeremove')
beforeremove
;;
*)
esac
